#!/usr/bin/python
###########################################################################
#
# Bowe Strickland <bowe@yak.net> Copyright 2005
# $Id: XMLPodChannel.py,v 1.3 2005/12/10 10:40:35 bowe Exp $
#
###########################################################################

import sys, os, string, re

from libxml2 import parseMemory
from XMLSimpleElement import XMLSimpleElement, factory

################################################################33
#
# utility 
#
################################################################33

import time
invalid_re = re.compile("\W+")

def parse_time_str(timestr, informats=[], outformat=None):

	"""
		Given a textual timestamps, try to apply an array of 
		strptime formats until we have a hit.  Then reformat
		string w/ specified output.  If no formats
		apply, at least return the timestamp with 
		non alphanumerics replaces with an _.
	"""

	outformat = outformat or "%Y-%m-%d_%H%M"

	if not informats: 
		informats = [	
			"%a, %d %b %Y %H:%M:%S %Z",			# NPR
		]

	#if not outformat: 
	#	outformat = "%Y-%m-%d_%H%M"

	tval = None
	for f in informats:
		try: 
			tval = time.strptime(timestr, f)
			if tval: break
		except ValueError: pass

	if tval:
		return time.strftime(outformat, tval)
	if not tval:
		return invalid_re.sub("_", timestr)[:25]

################################################################33
#
# Domish stuff
#
################################################################33

class RSS(XMLSimpleElement):
	name = "rss"
	complex_tags = "channel".split()

class Channel(XMLSimpleElement):
	name = "channel"
	simple_tags = """
		title link description copyright
		generator lastBuildDate 
		""".split()

	complex_tags = "image item".split()
	forcelist_tags = "item".split()

class Image(XMLSimpleElement):
	name = "image"
	simple_tags = "url title link".split()

class Item(XMLSimpleElement):
	name = "item"
	simple_tags = """
		title description link guid pubDate enclosure
		""".split()

factory.register(RSS)
factory.register(Channel)
factory.register(Image)
factory.register(Item)

################################################################33
#
# Pod Client
#
################################################################33

class XMLPodChannel(Channel):

	def __init__(self, url): 

		self.url = url.strip()

		# curl approach
		#import popen2
		#cmd = popen2.Popen3("curl %s" % self["url"])
		#cmd.wait()
		#content = cmd.fromchild.read()

		import urllib
		content = urllib.urlopen(self.url).read()

		doc = parseMemory(content, len(content))
		channel = doc.xpathEval("//channel")[0]

		Channel.__init__(self, channel)

	def get_suggested_filenames(self):

		r = []
		if not self["item"]: return 

		for item in self["item"]:

			enc = item.get("enclosure")
			if not enc: continue

			title = item.get("title")
			if title != None: title = title.content
			else: title = "no title"

			pubdate = item.get("pubDate")
			if pubdate != None: pubdate = pubdate.content
			else: pubdate = "no pubdate"

			t = invalid_re.sub("_", title).strip()
			d = parse_time_str(pubdate)

			a = ( enc["@url"], "%s.%s.mp3" % (t, d),
				   enc["@type"], enc["@length"],)

			r.append(a)

		return r

if __name__ == '__main__':

	feeds = """
		http://www.npr.org/rss/podcast.php?id=510024&uid=c3a3a5d6768ad165e5c38dc243599437
		http://streams.wgbh.org/rss/rss.php?feed=blues
	""".split()

	channels = [ XMLPodChannel(f) for f in feeds ]
	#for c in channels: print str(c.reduce_to_hash())
	for c in channels: print c.get_suggested_filenames()

#
# vi: ts=4
#


