#!/usr/bin/python
###########################################################################
#
# Bowe Strickland <bowe@yak.net> Copyright 2005
# $Id: XMLSimpleElement.py,v 1.2 2005/12/10 10:40:35 bowe Exp $
#
# utility which converst XML tree to python hash
# (attributes currently ignored)
#
###########################################################################

import sys, os, string, re
from UserDict import DictMixin

class XMLSimpleElementFactory:

	known_classes = {}

	def new(self, node, type=None):
		if not type: type = node.name
		f = self.known_classes[type]
		return f(node)

	def register(self, klass, name=None): 
		key = name or klass.name
		self.known_classes[key] = klass

factory = XMLSimpleElementFactory()

class XMLSimpleElement(DictMixin):

	name = "abstract superclass"
	simple_tags = []
	complex_tags = []
	forcelist_tags = []

	def __init__(self, node, name="", content=""):

		self.node = node
		self.content = content
		if name: self.name = name

		self.children = {}
		self.attrs = {}

		self._parse_attributes()
		self._parse_simple_tags()
		self._parse_complex_tags()

#		if self.content: self["@_content"] = content

	def __getitem__(self, key): 

		if key[0] == "@": return self.attrs[key]
		return self.children[key]

	def __setitem__(self, key, value): 

		if key[0] == "@": self.attrs[key] = value
		else: self.children[key] = value

	def __delitem__(self, key): 
		if key[0] == "@": del self.attrs[key]
		else: del self.children[key]

	def keys(self): 
		return self.children.keys() + self.attrs.keys()

	def _parse_attributes(self):
		a = self.node.properties
		while a:
			self["@"+a.name] = a.content
			a = a.next

	def _parse_simple_tags(self):

		for t in self.simple_tags:

			hits = self.node.xpathEval("./%s" % t)
			if not hits: continue

			self[t] = [ 
				XMLSimpleElement(h, name=t, content=h.content) for h in hits ]


			if len(self[t]) == 1 and t not in self.forcelist_tags:
				self[t] = self[t][0]


	def _parse_complex_tags(self):

		for k in self.complex_tags:

			hits = self.node.xpathEval("./%s" % k)
			if not hits: continue

			l = [ factory.new(i) for i in hits ]
			if len(l) == 1 and k not in self.forcelist_tags: self[k] = l[0]
			else: self[k] =  l

	def __str__(self):

		if not self.attrs: a = ""
		else:
			a = ['%s="%s"' % (k[1:], v) for k,v in self.attrs.items()]
			a = " " + " ".join(a)
		if not self.content and not self.children:
			return "<%s%s/>" % (self.name, a)

		if self.content: b = self.content
		else: 
			b = []
			for i in self.values():
				if type(i) == type([]):
					b.append("".join([ str(j) for j in i ]))
				else: b.append(str(i))
			b = "".join(b)

		return "<%s%s>%s</%s>" % (self.name, a, b, self.name)

	def reduce_to_hash(self):
		h = {}
		h.update(self.attrs)
		if self.content: 
			if self.has_key("content"):
				raise Exception, "syntax problem: attribute 'content' exists"
			h["content"] = self.content
		for k,v in self.children.items(): 
			if type(v) == type([]):
				h[k] = [ i.reduce_to_hash() for i in v ]
			else: h[k] = v.reduce_to_hash()

		return h

if __name__ == '__main__':

	import libxml2

	class Food(XMLSimpleElement):
		name = "food"
		simple_tags = "dessert".split()
		complex_tags = "fruit".split()

	class Fruit(XMLSimpleElement):
		name = "fruit"
		simple_tags = "apple".split()
		makelist_tags = "apple".split()

	factory.register(Food)
	factory.register(Fruit)


	test = """
		<food good="kindof" quality="stale">
			<fruit tasty="yes">
				<apple peeled="no">gala</apple>
				<apple peeled="yes" cored="no">mcintosh</apple>
			</fruit>
			<fruit>
				<apple>red delicious</apple>
			</fruit>
			<dessert type="savory">cheese</dessert>
			<dessert type="sweet">pudding</dessert>
		</food>
	"""

	doc = libxml2.parseMemory(test, len(test))
	food = doc.xpathEval("//food")[0]

	p = Food(food)
	print str(p)
	print p.reduce_to_hash()

# vi: ts=4
